@@ -51,15 +51,18 @@ extern "C" {
 #define strcasecmp _stricmp
 #endif
 
-#define NUMSWAPS 50
 #define NUMVAR 50
-#define LETSIGNSIZE 128
+#define LETSIGNSIZE 256
+// noletsignbefore and noletsignafter is hardly ever used and usually
+// only with very few chars, so it only needs a small array
+#define LETSIGNBEFORESIZE 64
+#define LETSIGNAFTERSIZE 64
 #define SEQPATTERNSIZE 128
 #define CHARSIZE sizeof(widechar)
 #define DEFAULTRULESIZE 50
 
 typedef struct intCharTupple {
-	int key;
+	unsigned long long key;
 	char value;
 } intCharTupple;
 
@@ -72,9 +75,17 @@ typedef struct intCharTupple {
 #define MAX_EMPH_CLASSES 10  // {emph_1...emph_10} in typeforms enum (liblouis.h)
 
 typedef unsigned int TranslationTableOffset;
-#define OFFSETSIZE sizeof(TranslationTableOffset)
+
+/* Basic type for translation table data, which carries all alignment
+ * constraints that fields contained in translation table may have.
+ * Notably TranslationTableCharacterAttributes is unsigned long long, so we need
+ * at least this big basic type. */
+typedef unsigned long long TranslationTableData;
+#define OFFSETSIZE sizeof(TranslationTableData)
 
 typedef enum {
+	/* The first 8 are the predefined character classes. They need to be listed first and
+	   in this order because of how allocateCharacterClasses works. */
 	CTC_Space = 0x1,
 	CTC_Letter = 0x2,
 	CTC_Digit = 0x4,
@@ -84,32 +95,35 @@ typedef enum {
 	CTC_Math = 0x40,
 	CTC_Sign = 0x80,
 	CTC_LitDigit = 0x100,
-	CTC_Class1 = 0x200,
-	CTC_Class2 = 0x400,
-	CTC_Class3 = 0x800,
-	CTC_Class4 = 0x1000,
+	CTC_CapsMode = 0x200,
+	CTC_EmphMode = 0x400,
+	CTC_NumericMode = 0x800,
+	CTC_NumericNoContract = 0x1000,
 	CTC_SeqDelimiter = 0x2000,
 	CTC_SeqBefore = 0x4000,
 	CTC_SeqAfter = 0x8000,
-	CTC_UserDefined0 = 0x10000,  // class 5
-	CTC_UserDefined1 = 0x20000,
-	CTC_UserDefined2 = 0x40000,
-	CTC_UserDefined3 = 0x80000,
-	CTC_UserDefined4 = 0x100000,
-	CTC_UserDefined5 = 0x200000,
-	CTC_UserDefined6 = 0x400000,
-	CTC_UserDefined7 = 0x800000,  // class 12
-	CTC_CapsMode = 0x1000000,
-	CTC_EmphMode = 0x2000000,
-	CTC_NumericMode = 0x4000000,
-	CTC_NumericNoContract = 0x8000000,
-	CTC_EndOfInput = 0x10000000,  // only used by pattern matcher
-	CTC_EmpMatch = 0x20000000,	// only used in TranslationTableRule->before and
-								  // TranslationTableRule->after
-	CTC_MidEndNumericMode = 0x40000000,
-	// 33 more bits available in a unsigned long long (at least 64 bits)
-	// currently used for classes 13 to 45
-	CTC_Class13 = 0x80000000,
+	/* The following 8 are reserved for %0 to %7 (in no particular order) */
+	/* Be careful with changing these values (and also CTC_EndOfInput) because in
+	   pattern_compile_expression they are stored in a unsigned int after cutting of the
+	   16 least significant bits. */
+	CTC_UserDefined1 = 0x10000,
+	CTC_UserDefined2 = 0x20000,
+	CTC_UserDefined3 = 0x40000,
+	CTC_UserDefined4 = 0x80000,
+	CTC_UserDefined5 = 0x100000,
+	CTC_UserDefined6 = 0x200000,
+	CTC_UserDefined7 = 0x400000,
+	CTC_UserDefined8 = 0x800000,
+	CTC_EndOfInput = 0x1000000,  // only used by pattern matcher
+	CTC_EmpMatch = 0x2000000,	// only used in TranslationTableRule->before and
+								 // TranslationTableRule->after
+	CTC_MidEndNumericMode = 0x4000000,
+	/* At least 37 more bits available in a unsigned long long (at least 64 bits). Used
+	   for custom attributes 9 to 45. These need to be the last values of the enum. */
+	CTC_UserDefined9 = 0x8000000,
+	CTC_UserDefined10 = 0x10000000,
+	CTC_UserDefined11 = 0x20000000,
+	CTC_UserDefined12 = 0x40000000,
 } TranslationTableCharacterAttribute;
 
 typedef enum {
@@ -185,7 +199,7 @@ typedef struct {
 	TranslationTableOffset next;
 	widechar lookFor;
 	widechar found;
-} CharOrDots;
+} CharDotsMapping;
 
 typedef struct {
 	TranslationTableOffset next;
@@ -284,6 +298,7 @@ typedef enum { /* Op codes */
 	CTO_Pass4,
 	CTO_Repeated,
 	CTO_RepWord,
+	CTO_RepEndWord,
 	CTO_CapsNoCont,
 	CTO_Always,
 	CTO_ExactDots,
@@ -440,7 +455,8 @@ typedef struct {
 	TranslationTableCharacterAttributes after;  /** character types which must follow */
 	TranslationTableCharacterAttributes before; /** character types which must precede */
 	TranslationTableOffset patterns;			/** before and after patterns */
-	TranslationTableOpcode opcode;		 /** rule for testing validity of replacement */
+	TranslationTableOpcode opcode; /** rule for testing validity of replacement */
+	char nocross;
 	short charslen;						 /** length of string to be replaced */
 	short dotslen;						 /** length of replacement string */
 	widechar charsdots[DEFAULTRULESIZE]; /** find and replacement strings */
@@ -484,13 +500,28 @@ typedef struct {
 	TranslationTableOffset bytesUsed;
 	TranslationTableOffset charToDots[HASHNUM];
 	TranslationTableOffset dotsToChar[HASHNUM];
-	TranslationTableOffset ruleArea[1]; /** Space for storing all rules and values */
+	TranslationTableData ruleArea[1]; /** Space for storing all rules and values */
 } DisplayTableHeader;
 
 /**
  * Translation table header
  */
 typedef struct { /* translation table */
+
+	/* state needed during compilation */
+	TranslationTableOffset tableSize;
+	TranslationTableOffset bytesUsed;
+	CharacterClass *characterClasses;
+	TranslationTableCharacterAttributes nextCharacterClassAttribute;
+	TranslationTableCharacterAttributes nextNumberedCharacterClassAttribute;
+	RuleName *ruleNames;
+	TranslationTableCharacterAttributes
+			numberedAttributes[8]; /* attributes 0-7 used in match rules (could also be
+								   stored in `characterClasses', but this is slightly
+								   faster) */
+	int usesAttributeOrClass;	  /* 1 = attribute, 2 = class */
+
+	/* needed for translation or other api functions */
 	int capsNoCont;
 	int numPasses;
 	int corrections;
@@ -498,8 +529,6 @@ typedef struct { /* translation table */
 	int usesSequences;
 	int usesNumericMode;
 	int usesEmphMode;
-	TranslationTableOffset tableSize;
-	TranslationTableOffset bytesUsed;
 	TranslationTableOffset undefined;
 	TranslationTableOffset letterSign;
 	TranslationTableOffset numberSign;
@@ -509,42 +538,24 @@ typedef struct { /* translation table */
 	int seqPatternsCount;
 	widechar seqAfterExpression[SEQPATTERNSIZE];
 	int seqAfterExpressionLength;
-
-	/* emphRules, including caps. */
-	TranslationTableOffset emphRules[MAX_EMPH_CLASSES + 1][9];
-
-	/* state needed during compilation */
-	CharacterClass *characterClasses;
-	TranslationTableCharacterAttributes nextCharacterClassAttribute;
-	RuleName *ruleNames;
-
+	TranslationTableOffset emphRules[MAX_EMPH_CLASSES + 1][9]; /* includes caps */
 	TranslationTableOffset begComp;
-	TranslationTableOffset compBegEmph1;
-	TranslationTableOffset compEndEmph1;
-	TranslationTableOffset compBegEmph2;
-	TranslationTableOffset compEndEmph2;
-	TranslationTableOffset compBegEmph3;
-	TranslationTableOffset compEndEmph3;
-	TranslationTableOffset compCapSign;
-	TranslationTableOffset compBegCaps;
-	TranslationTableOffset compEndCaps;
 	TranslationTableOffset endComp;
 	TranslationTableOffset hyphenStatesArray;
-	widechar noLetsignBefore[LETSIGNSIZE];
+	widechar noLetsignBefore[LETSIGNBEFORESIZE];
 	int noLetsignBeforeCount;
 	widechar noLetsign[LETSIGNSIZE];
 	int noLetsignCount;
-	widechar noLetsignAfter[LETSIGNSIZE];
+	widechar noLetsignAfter[LETSIGNAFTERSIZE];
 	int noLetsignAfterCount;
 	TranslationTableOffset characters[HASHNUM]; /** Character definitions */
 	TranslationTableOffset dots[HASHNUM];		/** Dot definitions */
 	TranslationTableOffset compdotsPattern[256];
-	TranslationTableOffset swapDefinitions[NUMSWAPS];
 	TranslationTableOffset forPassRules[MAXPASS + 1];
 	TranslationTableOffset backPassRules[MAXPASS + 1];
 	TranslationTableOffset forRules[HASHNUM];  /** chains of forward rules */
 	TranslationTableOffset backRules[HASHNUM]; /** Chains of backward rules */
-	TranslationTableOffset ruleArea[1]; /** Space for storing all rules and values */
+	TranslationTableData ruleArea[1]; /** Space for storing all rules and values */
 } TranslationTableHeader;
 
 typedef enum {
@@ -648,7 +659,7 @@ _lou_getDotsForChar(widechar c, const DisplayTableHeader *table);
  * TODO: move to commonTranslationFunctions.c
  */
 widechar EXPORT_CALL
-_lou_getCharFromDots(widechar d, const DisplayTableHeader *table);
+_lou_getCharForDots(widechar d, const DisplayTableHeader *table);
 
 void EXPORT_CALL
 _lou_getTable(const char *tableList, const char *displayTableList,
@@ -784,14 +795,15 @@ _lou_handlePassVariableAction(const widechar *instructions, int *IC);
 
 int EXPORT_CALL
 _lou_pattern_compile(const widechar *input, const int input_max, widechar *expr_data,
-		const int expr_max, const TranslationTableHeader *t);
+		const int expr_max, TranslationTableHeader *table, const FileInfo *nested);
 
 void EXPORT_CALL
 _lou_pattern_reverse(widechar *expr_data);
 
 int EXPORT_CALL
 _lou_pattern_check(const widechar *input, const int input_start, const int input_minmax,
-		const int input_dir, const widechar *expr_data, const TranslationTableHeader *t);
+		const int input_dir, const widechar *expr_data,
+		const TranslationTableHeader *table);
 
 /**
  * Read a line of widechar's from an input file
